﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;

namespace RSTextMonitor
{
    class BotTools
    {
        ScreenCapture sc { get; set; }
        public BotTools()
        {
            this.sc = new ScreenCapture();
        }
        public BotTools(ScreenCapture sc)
        {
            this.sc = sc;
        }

        #region MSfunctions

        [DllImport("User32.dll")]
        public static extern IntPtr GetDesktopWindow();
        [DllImport("User32.dll")]
        public static extern IntPtr GetWindowDC(IntPtr hWnd);
        [DllImport("Gdi32.dll")]
        public static extern int GetPixel(IntPtr hdc, int nXPos, int nYPos);
        #endregion

        #region Time
        public static long unixTime()
        {
            return (long)((DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds);
        }

        public static int secondsOfDay()
        {
            return Int32.Parse(DateTime.Now.ToString("HH")) * 3600 + Int32.Parse(DateTime.Now.ToString("mm")) * 60 + Int32.Parse(DateTime.Now.ToString("ss"));
        }
        #endregion

        #region Color detection methods

        public Point GetGameLocation()
        {
            Bitmap bmp = sc.CaptureAllScreens();
            for (int x = 0; x < bmp.Width - 764; x++)
            {
                for (int y = 0; y < bmp.Height - 502; y++)
                {
                    if (BotTools.isTolerant(Color.FromArgb(25, 19, 5), bmp.GetPixel(x, y), 5) &&
                        BotTools.isTolerant(Color.FromArgb(91, 75, 51), bmp.GetPixel(x, y + 502), 5) &&
                        BotTools.isTolerant(Color.FromArgb(18, 21, 12), bmp.GetPixel(x + 764, y + 502), 5))
                    {
                        return new Point(x, y);
                    }
                }
            }
            for (int x = 0; x < bmp.Width - 764; x++)
            {
                for (int y = 0; y < bmp.Height - 502; y++)
                {
                    if (BotTools.isTolerant(Color.FromArgb(147, 137, 114), bmp.GetPixel(x + 2, y + 475), 5) &&
                        BotTools.isTolerant(Color.FromArgb(179, 176, 168), bmp.GetPixel(x + 3, y + 341), 5) &&
                        BotTools.isTolerant(Color.FromArgb(42, 36, 21), bmp.GetPixel(x + 6, y + 344), 5) &&
                        BotTools.isTolerant(Color.FromArgb(7, 5, 4), bmp.GetPixel(x + 2, y + 338), 5))
                    {
                        return new Point(x, y);
                    }
                }
            }

            return new Point(-1, -1);
        }

        public static bool isTolerant(Color one, Color two, byte tolerance)
        {
            return (Math.Abs(one.B - two.B) <= tolerance &&
                    Math.Abs(one.G - two.G) <= tolerance &&
                    Math.Abs(one.R - two.R) <= tolerance);
        }

        //returns true if success, false if timed out
        public bool waitForPx(int x, int y, Color waitColor, byte tolerance, int timeout, bool positive)
        {
            long startsec = unixTime();
            while (positive != isTolerant(waitColor, sc.GetColorOfPx(x, y), tolerance))
            {
                if (unixTime() - startsec >= timeout)
                    return false;
                Thread.Sleep(400);
            }
            return true;
        }

        public bool waitForAreaChange(Point p1, Point p2, byte tolerance, int pixelreq, int timeout)
        {
            int pixelfound = 0;
            long startsec = unixTime();

            Bitmap bmp = sc.CaptureScreenArea(p1.X, p1.Y, p2.X - p1.X, p2.Y - p1.Y);
            Color pix;

            Bitmap checkbmp;
            Color checkpix;

            while (true)
            {
                checkbmp = sc.CaptureScreenArea(p1.X, p1.Y, p2.X - p1.X, p2.Y - p1.Y);
                for (int y = 0; y < p2.Y - p1.Y; y++)
                {
                    for (int x = 0; x < p2.X - p1.X; x++)
                    {
                        pix = bmp.GetPixel(x, y);
                        checkpix = checkbmp.GetPixel(x, y);

                        if (!isTolerant(pix, checkpix, tolerance))
                        {
                            pixelfound++;
                            if (pixelfound >= pixelreq)
                            {
                                checkbmp.Dispose();
                                bmp.Dispose();
                                return true;
                            }
                        }

                        if (unixTime() - startsec >= timeout)
                        {
                            checkbmp.Dispose();
                            bmp.Dispose();
                            return false;
                        }
                    }

                    Thread.Sleep(2);
                }
                pixelfound = 0;
                checkbmp.Dispose();

                Thread.Sleep(100);
            }
        }

        public bool includesColor(Point p1, Point p2, Color checkColor, byte tolerance)
        {
            Bitmap bmp = sc.CaptureScreenArea(p1.X, p1.Y, p2.X - p1.X, p2.Y - p1.Y);
            for (int y = 0; y < p2.Y - p1.Y; y++)
            {
                for (int x = 0; x < p2.X - p1.X; x++)
                {
                    if (isTolerant(checkColor, bmp.GetPixel(x, y), tolerance))
                    {
                        bmp.Dispose();
                        return true;
                    }
                }
            }
            bmp.Dispose();
            return false;
        }
        #endregion
    }
}
