﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Media;
using System.Diagnostics;

namespace RSTextMonitor
{
    public partial class Form1 : Form
    {
        Thread RunThread { get; set; }
        Thread TimerThread { get; set; }
        Thread SlayerXPWatcherThread { get; set; }

        volatile int CannonTimeLeft = -1;
        volatile int TaskTimeLeft = -1;
        string SlayerTask { get; set; }

        Point RS = new Point(-1, -1);
        SoundPlayer player { get; set; }

        BotTools bt { get; set; }
        ScreenCapture sc { get; set; }

        TextReader reader { get; set; }

        List<DataPoint> DataPoints { get; set; }

        string SlayerHelperFolder { get; set; }
        string PreferencesFile { get; set; }

        struct DataPoint
        {
            public long Time;
            public int XP;
            public string Task;
            public DataPoint(long Time, int XP, string Task)
            {
                this.Time = Time;
                this.XP = XP;
                this.Task = Task;
            }


            public static DataPoint GetDataPoint(string JSStatement)
            {
                int start = JSStatement.IndexOf('(') + 1;
                JSStatement = JSStatement.Substring(start, JSStatement.IndexOf(')') - start);
                string[] paramz = JSStatement.Split(',');
                paramz[2] = paramz[2].Replace("'", "");
                return new DataPoint(Int64.Parse(paramz[0]), Int32.Parse(paramz[1]), paramz[2] == "null" ? null : paramz[2]);
            }
        }

        public Form1()
        {
            InitializeComponent();
            sc = new ScreenCapture();
            bt = new BotTools(sc);
            reader = new TextReader();
            DataPoints = new List<DataPoint>();
            player = new SoundPlayer();
            RunThread = new Thread(Run);
            TimerThread = new Thread(Timer);
            SlayerXPWatcherThread = new Thread(SlayerXPWatcher);

            if (File.Exists("content\\graph\\datapoints.js"))
            {
                string[] Lines = File.ReadAllLines("content\\graph\\datapoints.js");

                for (int i = 2; i < Lines.Length; i += 2)
                {
                    DataPoints.Add(DataPoint.GetDataPoint(Lines[i]));
                }
            }

            SlayerHelperFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "CML\\SlayerHelper");
            PreferencesFile = Path.Combine(SlayerHelperFolder, "preferences.txt");

            if (File.Exists(PreferencesFile))
            {
                foreach (string Line in File.ReadAllLines(PreferencesFile))
                {
                    try
                    {
                        int value = Int32.Parse(Line.Substring(Line.IndexOf('=') + 1));
                        switch (Line.Substring(0, Line.IndexOf('=')))
                        {
                            case "stayontop":
                                stayOnTopToolStripMenuItem.Checked = (value == 1);
                                this.TopMost = stayOnTopToolStripMenuItem.Checked;
                                break;
                            case "minimizetotray":
                                minimizeToTrayToolStripMenuItem.Checked = (value == 1);
                                break;
                            case "playsound":
                                playSoundToolStripMenuItem.Checked = (value == 1);
                                playSoundToolStripMenuItem_Click(null, null);
                                break;
                            case "cannon_2min":
                                cannon2minToolStripMenuItem.Checked = (value == 1);
                                break;
                            case "cannon_ammo":
                                cannonammoToolStripMenuItem.Checked = (value == 1);
                                break;
                            case "no_prayer":
                                noprayerToolStripMenuItem.Checked = (value == 1);
                                break;
                            case "task_complete":
                                taskcompleteToolStripMenuItem.Checked = (value == 1);
                                break;
                        }
                    }
                    catch { }
                }
            }

            FindButton_Click(null, null);

            RunThread.Start();
            TimerThread.Start();
            SlayerXPWatcherThread.Start();
        }

        static string Pad(int i)
        {
            if (i < 10)
            {
                return "0" + i;
            }
            else
            {
                return i.ToString();
            }
        }

        void SlayerXPWatcher()
        {
            while (true)
            {
                if (bt.waitForPx(RS.X + 617, RS.Y + 429, Color.FromArgb(255, 255, 160), 5, 5, true))
                {
                    Bitmap bmp = sc.CaptureScreenArea(RS.X, RS.Y, 764, 502);
                    for (int x = 600; x < 640; x++)
                    {
                        if (BotTools.isTolerant(Color.FromArgb(255, 255, 160), bmp.GetPixel(x, 441), 5))
                        {
                            string s = reader.ReadText(x, 441, 120, bmp, Color.Black, false);
                            if (s.StartsWith("Slayer"))
                            {
                                string numstr = "";
                                foreach (char c in s)
                                {
                                    if (47 < (byte)c && (byte)c < 58)
                                    {
                                        numstr += c;
                                    }
                                }
                                if (numstr != "")
                                {
                                    DataPoint dp = new DataPoint(BotTools.unixTime(), Int32.Parse(numstr), SlayerTask);
                                    DataPoints.Add(dp);
                                    DataPoint recentdp = DataPoints[Math.Max(0, DataPoints.Count - 2)];

                                    DataPoint EarliestOnTask = DataPoints[0];
                                    if (SlayerTask != null)
                                    {
                                        for (int i = Math.Max(0, DataPoints.Count - 2); i > -1; i--)
                                        {
                                            if (DataPoints[i].Task != SlayerTask)
                                            {
                                                EarliestOnTask = DataPoints[i + 1];
                                                break;
                                            }
                                        }
                                    }


                                    if (dp.Time != DataPoints[0].Time)
                                    {
                                        if (dp.Time != recentdp.Time)
                                        {
                                            try
                                            {
                                                this.Invoke((MethodInvoker)delegate
                                                {
                                                    SlayerXPLabel.Text =
                                                        ((dp.Time == recentdp.Time) ? "" : ("Most recent: " + (dp.XP - recentdp.XP) + " XP/" + (dp.Time - recentdp.Time) + " sec (" +
                                                        ((dp.XP - recentdp.XP) * 3600 / (dp.Time - recentdp.Time)) + " XP/hour)")
                                                        ) + Environment.NewLine +
                                                        ((dp.Time == EarliestOnTask.Time) ? "" : ("This task: " +
                                                        (SlayerTask == null ? "no data" : ((dp.XP - EarliestOnTask.XP) + " XP/" + (dp.Time - EarliestOnTask.Time) + " sec (" +
                                                        ((dp.XP - EarliestOnTask.XP) * 3600 / (dp.Time - EarliestOnTask.Time)) + " XP/hour)")
                                                        )));
                                                });
                                            }
                                            catch { }
                                        }
                                    }
                                    else
                                    {
                                        try
                                        {
                                            this.Invoke((MethodInvoker)delegate
                                            {
                                                SlayerXPLabel.Text = "Hover over Slayer in skills tab again";

                                            });
                                        }
                                        catch { }
                                    }
                                }
                            }
                            break;
                        }
                    }
                    bmp.Dispose();
                    while (!bt.waitForPx(RS.X + 617, RS.Y + 429, Color.FromArgb(255, 255, 160), 5, 5, false)) ;
                }
            }
        }

        void Timer()
        {
            while (true)
            {
                Thread.Sleep(1000);
                if (CannonTimeLeft > 0)
                {
                    CannonTimeLeft--;
                    try
                    {
                        this.Invoke((MethodInvoker)delegate
                        {
                            TimeLabel.Text = "Cannon " + (CannonTimeLeft / 60) + ":" + Pad(CannonTimeLeft % 60);
                        });
                    }
                    catch { }

                    if (CannonTimeLeft == 120)
                    {
                        PlaySound("cannon_2min");
                    }
                }

                if (TaskTimeLeft > 0)
                {
                    TaskTimeLeft--;
                    try
                    {
                        this.Invoke((MethodInvoker)delegate
                        {
                            TaskTimeLabel.Text = "Task ETC " + (TaskTimeLeft / 60) + ":" + Pad(TaskTimeLeft % 60);
                        });
                    }
                    catch { }
                }
            }
        }

        void Run()
        {
            string[] PreviousLines = new string[8];
            string[] CurrentLines = new string[8];
            int PreviousKills = 0;
            long PreviousTime = BotTools.unixTime();

            int CurrentKills = 0;
            long CurrentTime = PreviousTime;

            while (true)
            {
                List<string> NewLines = new List<string>();
                Bitmap bmp = sc.CaptureScreenArea(RS.X, RS.Y, 764, 502);

                if (BotTools.isTolerant(bmp.GetPixel(504, 348), Color.FromArgb(0, 0, 1), 5) && !BotTools.isTolerant(bmp.GetPixel(511, 442), Color.FromArgb(61, 52, 38), 5))
                {
                    for (int i = 0; i < 8; i++)
                    {
                        CurrentLines[7 - i] = reader.ReadText(12, i * 14 + 356, 484, bmp, Color.Black, false);
                    }
                    
                    for (int i = 0; i < 8; i++)
                    {
                        bool IsMatch = true;
                        for (int k = 0; k < 8 - i; k++)
                        {
                            if (CurrentLines[k + i] != PreviousLines[k])
                            {
                                IsMatch = false;
                                break;
                            }
                        }

                        if (IsMatch)
                        {
                            for (int k = i - 1; k > -1; k--)
                            {
                                NewLines.Add(CurrentLines[k]);
                            }
                            break;
                        }
                    }

                    foreach (string s in NewLines)
                    {
                        //Process new messages appeared in chat box
                        if (s == "Youplacethecannonbaseontheground")
                        {
                            CannonTimeLeft = 1500;
                        }
                        else if (s.StartsWith("Youpickupthecannon"))
                        {
                            CannonTimeLeft = 1;
                        }
                        else if (s.StartsWith("Yourcannonhasalmostdecayed"))
                        {
                            CannonTimeLeft = 300;
                        }
                        else if (s.StartsWith("Youvecompletedyourtask"))
                        {
                            PreviousKills = 0;
                            PreviousTime = BotTools.unixTime();

                            CurrentKills = 0;
                            CurrentTime = PreviousTime;

                            SlayerTask = null;

                            DateTime now = DateTime.Now;
                            try
                            {
                                this.Invoke((MethodInvoker)delegate
                                {
                                    LastTaskLabel.Text = "Last task completed at " + now.Hour + ":" + Pad(now.Minute);
                                    TaskProgressLabel.Text = "Check kills on slayer gem";
                                    SlayerTaskLabel.Text = "Task: N/A";
                                });
                            }
                            catch { }
                            PlaySound("task_complete");
                        }
                        else if (s.StartsWith("Youreassignedtokill"))
                        {
                            try
                            {
                                string numstr = "";
                                foreach (char c in s)
                                {
                                    if (47 < (byte)c && (byte)c < 58)
                                    {
                                        numstr += c;
                                    }
                                }
                                PreviousKills = CurrentKills;
                                PreviousTime = CurrentTime;

                                CurrentKills = Int32.Parse(numstr);
                                CurrentTime = BotTools.unixTime();

                                if (CurrentTime != PreviousTime)
                                {
                                    if (PreviousKills != 0)
                                    {
                                        int KillsPerHour = (int)((PreviousKills - CurrentKills) * 3600 / (CurrentTime - PreviousTime));
                                        try
                                        {
                                            this.Invoke((MethodInvoker)delegate
                                            {
                                                TaskProgressLabel.Text = (PreviousKills - CurrentKills) + " kills/" + (CurrentTime - PreviousTime) + " sec (" + KillsPerHour + " kills/hour)";
                                            });
                                        }
                                        catch { }
                                        if (KillsPerHour != 0)
                                        {
                                            TaskTimeLeft = (int)(3600 * CurrentKills / KillsPerHour);
                                        }
                                    }
                                    else
                                    {
                                        try
                                        {
                                            this.Invoke((MethodInvoker)delegate
                                            {
                                                TaskProgressLabel.Text = "Check kills on slayer gem" + (CurrentKills != 0 ? " again" : "");
                                            });
                                        }
                                        catch { }
                                    }
                                }
                            }
                            catch (FormatException)
                            { }

                            SlayerTask = s.Substring(19, s.IndexOf("only") - 19);

                            this.Invoke((MethodInvoker)delegate
                            {
                                SlayerTaskLabel.Text = "Task: '" + SlayerTask + "'";
                            });
                        }
                        else if (s.StartsWith("Youhaverunoutofprayerpoints"))
                        {
                            PlaySound("no_prayer");
                        }
                        else if (s.StartsWith("Yourcannonisoutofammo"))
                        {
                            PlaySound("cannon_ammo");
                        }
                        else if (s.StartsWith("Youneedsomethingnewtohunt"))
                        {
                            SlayerTask = null;
                        }
                    }

                    for (int i = 0; i < 8; i++)
                    {
                        PreviousLines[i] = CurrentLines[i];
                    }
                }
                bmp.Dispose();
                bt.waitForAreaChange(new Point(RS.X + 8, RS.Y + 346), new Point(RS.X + 250, RS.Y + 459), 5, 5, 4);
            }
        }

        void PlaySound(string FileName)
        {
            if (playSoundToolStripMenuItem.Checked)
            {
                foreach (ToolStripMenuItem c in playSoundToolStripMenuItem.DropDownItems)
                {
                    if (c.Checked && c.Text == FileName)
                    {
                        try
                        {
                            player.SoundLocation = "content/sound/" + FileName + ".wav";
                            player.Play();
                        }
                        catch (InvalidOperationException)
                        {
                            MessageBox.Show("content/sound/" + FileName + ".wav must be a PCM .WAV file.");
                        }
                        catch (FileNotFoundException)
                        {
                            MessageBox.Show("content/sound/" + FileName + ".wav does not exist.");
                        }
                    }
                }
            }
        }

        private void FindButton_Click(object sender, EventArgs e)
        {
            Point p = bt.GetGameLocation();
            if (p.X != -1 && p.Y != -1)
            {
                RS = p;
                if (XBox.Text != p.X.ToString())
                    XBox.Text = p.X.ToString();
                if (YBox.Text != p.Y.ToString())
                    YBox.Text = p.Y.ToString();
            }
            else
            {
                MessageBox.Show("Your RuneScape game client could not be located. Make sure your entire game screen is visible and you are logged in. Then, click the Find button again.");
            }
        }

        private void XBox_TextChanged(object sender, EventArgs e)
        {
            RS.X = Int32.Parse(XBox.Text);
        }

        private void YBox_TextChanged(object sender, EventArgs e)
        {
            RS.Y = Int32.Parse(YBox.Text);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (RunThread != null)
            {
                RunThread.Abort();
                RunThread = null;
            }
            if (TimerThread != null)
            {
                TimerThread.Abort();
                TimerThread = null;
            }
            if (SlayerXPWatcherThread != null)
            {
                SlayerXPWatcherThread.Abort();
                SlayerXPWatcherThread = null;
            }
            SystemTrayIcon.Dispose();

            viewGraphAndXPRatesToolStripMenuItem_Click(null, null);
            try
            {
                string[] Lines = new string[]
                {
                    "stayontop=" + (stayOnTopToolStripMenuItem.Checked?"1":"0"),
                    "minimizetotray=" + (minimizeToTrayToolStripMenuItem.Checked?"1":"0"),
                    "playsound=" + (playSoundToolStripMenuItem.Checked?"1":"0"),
                    "cannon_2min=" + (cannon2minToolStripMenuItem.Checked?"1":"0"),
                    "cannon_ammo=" + (cannonammoToolStripMenuItem.Checked?"1":"0"),
                    "no_prayer=" + (noprayerToolStripMenuItem.Checked?"1":"0"),
                    "task_complete=" + (taskcompleteToolStripMenuItem.Checked?"1":"0")
                };
                if (!Directory.Exists(SlayerHelperFolder))
                {
                    Directory.CreateDirectory(SlayerHelperFolder);
                }

                File.WriteAllLines(PreferencesFile, Lines);
            }
            catch { }
        }

        private void SystemTrayIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            SystemTrayIcon.Visible = false;
            this.Show();
            this.WindowState = FormWindowState.Normal;
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (minimizeToTrayToolStripMenuItem.Checked)
            {
                if (FormWindowState.Minimized == this.WindowState)
                {
                    this.Hide();
                    SystemTrayIcon.Visible = true;
                }
                else if (FormWindowState.Normal == this.WindowState)
                {
                    SystemTrayIcon.Visible = false;
                    this.Show();
                }
            }
        }

        private void stayOnTopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.TopMost = stayOnTopToolStripMenuItem.Checked;
        }

        private void playSoundToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (ToolStripMenuItem c in playSoundToolStripMenuItem.DropDownItems)
            {
                c.Enabled = playSoundToolStripMenuItem.Checked;
            }
        }

        private void viewGraphAndXPRatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            List<string> Lines = new List<string>();
            Lines.Add("function addData(xpdata,arr){");
            int i = 0;
            foreach (DataPoint dp in DataPoints)
            {
                DateTime dt = new DateTime(1970, 1, 1, 0, 0, 0, 0);
                dt = dt.AddSeconds(dp.Time).ToLocalTime();
                Lines.Add("xpdata.addRow([new Date(" + dp.Time + "000)," + dp.XP + ",\"<span class='tooltip'><strong>" +
                    dt.ToShortDateString() + " " + dt.ToLongTimeString() + "</strong></br> XP:" + dp.XP + " Task: " +
                    (dp.Task == null ? "unknown" : dp.Task) + "</span>\"]);");
                Lines.Add("arr[" + i + "] = new Array(" + dp.Time + "," + dp.XP + "," +
                    (dp.Task == null ? "null" : "'" + dp.Task + "'") + ");");
                i++;
            }
            Lines.Add("}");
            File.WriteAllLines("content/graph/datapoints.js", Lines.ToArray());
            if (sender != null)
            {
                Process.Start("Graph.html");
            }
        }

        private void crystalMathLabsXPTrackerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://crystalmathlabs.com/tracker/currenttop.php?skill=19");
        }

        private void submitBugReportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://crystalmathlabs.com/tracker/contact.php");
        }
    }
}
